/***************************************************************************
 *   Copyright (C) 2007 by David Bitseff                                   *
 *   dbitsef@zipcon.net                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "directorylistview.h"
#include "logger.h"
#include <QDirModel>

DirectoryListView::DirectoryListView(QWidget* parent)
	: QListView(parent), viewer(0), viewerSize(320,200), toggle(true)
{
	dirTimer = new QTimer(this);
	connect(dirTimer, SIGNAL(timeout()), this, SLOT(refresh()));
	dirTimer->start(5000);
}

DirectoryListView::~DirectoryListView()
{
	if (viewer) delete viewer;
	dirTimer->stop();
	delete dirTimer;
}

void DirectoryListView::refresh()
{
	static QFileInfo last_m;
	if (isVisible())
	{
		QDirModel* dm = qobject_cast<QDirModel*>(model());
		QFileInfo info(dm->fileInfo(rootIndex()));
		info.refresh();

		if (last_m != info)
			last_m = info;

		else if (info.lastModified() > last_m.lastModified())
		{
			logFine(QString("DirectoryListView::refresh : updating view '%1' ")
					.arg(info.fileName()));
			last_m = info;
			dm->refresh();
		}
	}
}

void DirectoryListView::mousePressEvent(QMouseEvent* event)
{
	if (event->button() == Qt::MidButton)
		refresh();

	else if (event->button() == Qt::RightButton)
	{
		QModelIndex idx = indexAt(event->pos());
		if (idx.isValid())
		{
			QString s = qobject_cast<QDirModel*>
					(model())->fileInfo(idx).absoluteFilePath();
			s.replace(QRegExp("flam(3|e)?$"), "png");
			logFine(QString("DirectoryListView::mousePressEvent : showing %1")
					.arg(s));
			if (!viewer)
			{
				viewer = new MainViewer();
				connect(viewer, SIGNAL(viewerResized(const QSize&)),
						this, SLOT(viewerResizedAction(const QSize&)));
			}
			viewerImage = QPixmap(s);
			viewer->show();
			viewer->raise();
			viewer->setPixmap(viewerImage,false);
		}
	}
	else
		QListView::mousePressEvent(event);
}

void DirectoryListView::hideEvent(QHideEvent* e)
{
	if (viewer)
	{
		disconnect(viewer);
		viewer->close();
		delete viewer;
		viewer = 0;
	}
}

void DirectoryListView::viewerResizedAction(const QSize& s)
{
	if (viewer->isVisible())
		viewer->rescalePixmap();
}

void DirectoryListView::keyPressEvent(QKeyEvent* e)
{
	if (viewer && viewer->isVisible())
		viewer->keyPressEvent(e);
	QListView::keyPressEvent(e);
}


