/***************************************************************************
 *   Copyright (C) 2007 by David Bitseff                                   *
 *   dbitsef@zipcon.net                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QColorDialog>

#include "edittrianglewidget.h"
#include "xfedit.h"

#define genome_ptr (genome->selectedGenome())

EditTriangleWidget::EditTriangleWidget(GenomeVector* g, FigureEditor* e,
									   QWidget* parent)
	: QWidget(parent), genome(g), editor(e)
{
	setupUi(this);

	connect(m_rotateSlider, SIGNAL(valueChanged(int)),
            this, SLOT(triangleRotatedAction()));
	connect(m_scaleSlider, SIGNAL(valueChanged(int)),
            this, SLOT(triangleScaledAction()));
	connect(m_hFlipButton, SIGNAL(pressed()),
			this, SLOT(triangleHFlippedAction()));
	connect(m_vFlipButton, SIGNAL(pressed()),
			this, SLOT(triangleVFlippedAction()));
	connect(m_sceneScaleSlider, SIGNAL(valueChanged(int)),
			this, SLOT(sceneScaledAction()));
	connect(m_gridCheckBox, SIGNAL(toggled(bool)),
			this, SLOT(toggleGridAction(bool)));
	connect(m_gridColorButton, SIGNAL(pressed()),
			this, SLOT(gridColorSelectAction()));
	connect(m_bgColorButton, SIGNAL(pressed()),
			this, SLOT(bgColorSelectAction()));
	connect(m_moveLeftButton, SIGNAL(pressed()),
			this, SLOT(moveTriangleLeftAction()));
	connect(m_moveRightButton, SIGNAL(pressed()),
			this, SLOT(moveTriangleRightAction()));
	connect(m_moveUpButton, SIGNAL(pressed()),
			this, SLOT(moveTriangleUpAction()));
	connect(m_moveDownButton, SIGNAL(pressed()),
			this, SLOT(moveTriangleDownAction()));
	connect(m_rotateLineEdit, SIGNAL(returnPressed()),
			this, SLOT(rotateTextEditedSlot()));
	connect(m_scaleLineEdit, SIGNAL(returnPressed()),
			this, SLOT(scaleTextEditedSlot()));
	connect(m_markButton, SIGNAL(toggled(bool)),
			this, SLOT(toggleMarkAction(bool)));
}

void EditTriangleWidget::triangleScaledAction()
{
	int value = m_scaleSlider->dx();
	Qt::KeyboardModifiers mods = QApplication::keyboardModifiers();
	double scale;
	if (value > 0)
	{
		if (mods & Qt::ShiftModifier)
		{
			scale = 1.01;
			scale_dx *= 1.01;
		}
		else if (mods & Qt::ControlModifier)
		{
			scale = 1.10;
			scale_dx *= 1.10;
		}
		else
		{
			scale = 1.05;
			scale_dx *= 1.05;
		}
	}
	else if (value < 0)
	{
		if (mods & Qt::ShiftModifier)
		{
			scale = 0.99;
			scale_dx *= 0.99;
		}
		else if (mods & Qt::ControlModifier)
		{
			scale = 0.9090;
			scale_dx *= 0.9090;
		}
		else
		{
			scale = 0.9523;
			scale_dx *= 0.9523;
		}
	}
	else
		return;

	if (editor->hasSelection())
		editor->scaleSelection(scale, scale, selectionTransformPos());
	else
		editor->scaleTriangle(scale, scale,
				triangleTransformPos(), selectedTriangle);

	m_scaleLineEdit->setText(QString::number(scale_dx, 'f', 3));
}


void EditTriangleWidget::triangleRotatedAction()
{
	int value = m_rotateSlider->dx();
	if (value == 0)
		return;

	double div = 16.0;
	Qt::KeyboardModifiers mods = QApplication::keyboardModifiers();
	if (mods & Qt::ShiftModifier)
		div = 64.0;
	else if (mods & Qt::ControlModifier)
		div = 4.0;

	double deg = -1.0 * value / div;
	rotate_dx += deg;
	while (rotate_dx > 360.0)
		rotate_dx -= 360.0;
	while (rotate_dx < -360.0)
		rotate_dx += 360.0;
	m_rotateLineEdit->setText(QString::number(-1.0 * rotate_dx, 'f', 1));

	if (editor->hasSelection())
		editor->rotateSelection(deg, selectionTransformPos());
	else
		editor->rotateTriangle(deg, triangleTransformPos(),
				selectedTriangle);
}

void EditTriangleWidget::triangleHFlippedAction()
{
	if (editor->hasSelection())
		editor->flipSelectionHAction(selectionTransformPos());
	else
		editor->flipTriangleHAction(triangleTransformPos());
}

void EditTriangleWidget::triangleVFlippedAction()
{
	if (editor->hasSelection())
		editor->flipSelectionVAction(selectionTransformPos());
	else
		editor->flipTriangleVAction(triangleTransformPos());
}

void EditTriangleWidget::sceneScaledAction()
{
	int value = m_sceneScaleSlider->dx();
	Qt::KeyboardModifiers mods = QApplication::keyboardModifiers();
	double scale;
	if (value > 0)
	{
		if (mods & Qt::ShiftModifier)
		{
			scale = 1.01;
			scale_dx *= 1.01;
		}
		else if (mods & Qt::ControlModifier)
		{
			scale = 1.10;
			scale_dx *= 1.10;
		}
		else
		{
			scale = 1.05;
			scale_dx *= 1.05;
		}
	}
	else if (value < 0)
	{
		if (mods & Qt::ShiftModifier)
		{
			scale = 0.99;
			scale_dx *= 0.99;
		}
		else if (mods & Qt::ControlModifier)
		{
			scale = 0.9090;
			scale_dx *= 0.9090;
		}
		else
		{
			scale = 0.9523;
			scale_dx *= 0.9523;
		}
	}
	else
		return;

	editor->scaleBasis(scale, scale);
}

void EditTriangleWidget::toggleGridAction(bool checked)
{
	editor->setGridVisible(checked);
}

void EditTriangleWidget::showEvent(QShowEvent* e)
{
	// quack quack, better to just pass in the ref to the figureeditor...
	m_gridCheckBox->blockSignals(true);
	m_gridCheckBox->setChecked(editor->gridVisible());
	m_gridCheckBox->blockSignals(false);
}


void EditTriangleWidget::gridColorSelectAction()
{
	QColor c = QColorDialog::getColor(editor->gridColor(), this);
	if (c.isValid())
		editor->setGridColor(c);
}

void EditTriangleWidget::bgColorSelectAction()
{
	QColor c = QColorDialog::getColor(editor->bgColor(), this);
	if (c.isValid())
		editor->setbgColor(c);
}

void EditTriangleWidget::triangleSelectedSlot(Triangle* t)
{
	selectedTriangle = t;
	rotate_dx = 0.0;
	scale_dx  = 1.0;
	m_rotateLineEdit->setText("0.0");
	m_scaleLineEdit->setText("1.0");
}


void EditTriangleWidget::moveTriangleLeftAction()
{
	if (editor->hasSelection())
		editor->moveSelectionBy(m_multiplierEditor->value()*(-1.0), 0.0);
	else if (editor->postEnabled())
		editor->moveTriangleBy(m_multiplierEditor->value()*(-1.0), 0.0,
				editor->post());
	else
		editor->moveTriangleBy(m_multiplierEditor->value()*(-1.0), 0.0,
				selectedTriangle);
}


void EditTriangleWidget::moveTriangleRightAction()
{
	if (editor->hasSelection())
		editor->moveSelectionBy(m_multiplierEditor->value(),0.0);
	else if (editor->postEnabled())
		editor->moveTriangleBy(m_multiplierEditor->value(), 0.0, editor->post());
	else
		editor->moveTriangleBy(m_multiplierEditor->value(),0.0, selectedTriangle);
}



void EditTriangleWidget::moveTriangleUpAction()
{
	if (editor->hasSelection())
		editor->moveSelectionBy(0.0, m_multiplierEditor->value());
	else if (editor->postEnabled())
		editor->moveTriangleBy(0.0, m_multiplierEditor->value(), editor->post());
	else
		editor->moveTriangleBy(0.0, m_multiplierEditor->value(), selectedTriangle);
}



void EditTriangleWidget::moveTriangleDownAction()
{
	if (editor->hasSelection())
		editor->moveSelectionBy(0.0, m_multiplierEditor->value()*(-1.0));
	else if (editor->postEnabled())
		editor->moveTriangleBy(0.0, m_multiplierEditor->value()*(-1.0),
				editor->post());
	else
		editor->moveTriangleBy(0.0, m_multiplierEditor->value()*(-1.0),
				selectedTriangle);
}


void EditTriangleWidget::rotateTextEditedSlot()
{
	double deg = m_rotateLineEdit->text().toDouble();
	if (deg == 0.0)
		return;

	if (editor->hasSelection())
		editor->rotateSelection(deg, selectionTransformPos());
	else
		editor->rotateTriangle(deg, triangleTransformPos(), selectedTriangle);

	rotate_dx = 0.0;
}


void EditTriangleWidget::scaleTextEditedSlot()
{
	double scale = m_scaleLineEdit->text().toDouble();
	if (scale == 1.0)
		return;

	if (editor->hasSelection())
		editor->scaleSelection(scale, scale, selectionTransformPos());
	else
		editor->scaleTriangle(scale, scale,
				triangleTransformPos(), selectedTriangle);

	scale_dx = 0.0;
}


void EditTriangleWidget::toggleMarkAction(bool flag)
{
	editor->setMarkVisible(flag);
}


QPointF EditTriangleWidget::triangleTransformPos()
{
	Triangle* t = selectedTriangle;
	QPointF pos;

	if (editor->postEnabled())
		t = editor->post();

	if (m_triangleButton->isChecked())
	// get the triangle's center position from the scene
		pos = t->mapFromScene(t->sceneBoundingRect().center());
	else if (m_markButton->isChecked())
		// center on the marker
		pos = t->mapFromScene(editor->mark()->center());
	else if (m_nodeAButton->isChecked())
		pos = t->getCoords().at(0);
	else if (m_nodeBButton->isChecked())
		pos = t->getCoords().at(1);
	else if (m_nodeCButton->isChecked())
		pos = t->getCoords().at(2);

	return pos;
}


QPointF EditTriangleWidget::selectionTransformPos()
{
	QPointF pos(0.0, 0.0);
	if (m_triangleButton->isChecked())
		pos = editor->selection()->mapFromScene(
				editor->selection()->sceneBoundingRect().center());
	else if (m_markButton->isChecked())
		pos = editor->selection()->mapFromScene(editor->mark()->center());
	else
		pos = editor->selection()->mapFromScene(pos);

	return pos;
}


