/***************************************************************************
 *   Copyright (C) 2007 by David Bitseff                                   *
 *   dbitsef@zipcon.net                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "imgsettingswidget.h"
#include <QAction>

ImageSettingsWidget::ImageSettingsWidget(GenomeVector* gen, QWidget* parent)
	: QWidget(parent), genome(gen)
{
	setupUi(this);

	connect(m_qualityLineEdit, SIGNAL(valueUpdated()),
            this, SLOT(fieldEditedAction()));
	connect(m_filterLineEdit, SIGNAL(valueUpdated()),
            this, SLOT(fieldEditedAction()));
	connect(m_oversampleLineEdit, SIGNAL(valueUpdated()),
            this, SLOT(fieldEditedAction()));
	connect(m_batchesLineEdit, SIGNAL(valueUpdated()),
            this, SLOT(fieldEditedAction()));
	connect(m_temporalLineEdit, SIGNAL(valueUpdated()),
            this, SLOT(fieldEditedAction()));
	connect(m_estimatorLineEdit, SIGNAL(valueUpdated()),
            this, SLOT(fieldEditedAction()));
	connect(m_estimatorCurveLineEdit, SIGNAL(valueUpdated()),
            this, SLOT(fieldEditedAction()));
	connect(m_estimatorMinLineEdit, SIGNAL(valueUpdated()),
            this, SLOT(fieldEditedAction()));
	connect(m_symmetryLineEdit, SIGNAL(valueUpdated()),
            this, SLOT(fieldEditedAction()));
	connect(m_filterShapeBox, SIGNAL(activated(int)),
            this, SLOT(fieldEditedAction()));
	connect(m_applySymmetryButton, SIGNAL(pressed()),
			this, SLOT(applySymmetryAction()));
}

void ImageSettingsWidget::reset()
{
	updateFormData();
}

#define genome_ptr (genome->selectedGenome())


void ImageSettingsWidget::applySymmetryAction()
{
	int sym = m_symmetryLineEdit->value();
	if (sym == 1) return;
	flam3_add_symmetry(genome_ptr, sym);
	genome_ptr->symmetry = 1;
	m_symmetryLineEdit->setValue(1);
	emit symmetryAdded();
}

void ImageSettingsWidget::updateFormData()
{
	m_qualityLineEdit->updateValue(genome_ptr->sample_density);
	m_filterLineEdit->updateValue(genome_ptr->spatial_filter_radius);
	m_oversampleLineEdit->updateValue(genome_ptr->spatial_oversample);
	m_batchesLineEdit->updateValue(genome_ptr->nbatches);
	m_temporalLineEdit->updateValue(genome_ptr->ntemporal_samples);
	m_estimatorLineEdit->updateValue(genome_ptr->estimator);
	m_estimatorCurveLineEdit->updateValue(genome_ptr->estimator_curve);
	m_estimatorMinLineEdit->updateValue(genome_ptr->estimator_minimum);
	m_symmetryLineEdit->updateValue(genome_ptr->symmetry);

	// filter_shape
	QString fshape = "gaussian";
	if (genome_ptr->spatial_filter_select == flam3_gaussian_kernel)
		fshape ="gaussian";
	else if (genome_ptr->spatial_filter_select == flam3_hermite_kernel)
		fshape ="hermite";
	else if (genome_ptr->spatial_filter_select == flam3_box_kernel)
		fshape ="box";
	else if (genome_ptr->spatial_filter_select == flam3_triangle_kernel)
		fshape ="triangle";
	else if (genome_ptr->spatial_filter_select == flam3_bell_kernel)
		fshape ="bell";
	else if (genome_ptr->spatial_filter_select == flam3_b_spline_kernel)
		fshape ="bspline";
	else if (genome_ptr->spatial_filter_select == flam3_mitchell_kernel)
		fshape ="mitchell";
	else if (genome_ptr->spatial_filter_select == flam3_blackman_kernel)
		fshape ="blackman";
	else if (genome_ptr->spatial_filter_select == flam3_catrom_kernel)
		fshape ="catrom";
	else if (genome_ptr->spatial_filter_select == flam3_hanning_kernel)
		fshape ="hanning";
	else if (genome_ptr->spatial_filter_select == flam3_hamming_kernel)
		fshape ="hamming";
	else if (genome_ptr->spatial_filter_select == flam3_lanczos3_kernel)
		fshape ="lanczos3";
	else if (genome_ptr->spatial_filter_select == flam3_lanczos2_kernel)
		fshape ="lanczos2";
	else if (genome_ptr->spatial_filter_select == flam3_quadratic_kernel)
		fshape ="quadratic";
	m_filterShapeBox->setCurrentIndex(m_filterShapeBox->findText(fshape));
}

void ImageSettingsWidget::fieldEditedAction()
{
	genome_ptr->sample_density = qMax(0.00001, m_qualityLineEdit->value());
	genome_ptr->spatial_filter_radius = m_filterLineEdit->value();
	genome_ptr->spatial_oversample = m_oversampleLineEdit->value();
	genome_ptr->nbatches = m_batchesLineEdit->value();
	genome_ptr->ntemporal_samples = QString(m_temporalLineEdit->text()).toInt();
	genome_ptr->estimator = m_estimatorLineEdit->value();
	genome_ptr->estimator_curve = m_estimatorCurveLineEdit->value();
	genome_ptr->estimator_minimum = m_estimatorMinLineEdit->value();
	genome_ptr->symmetry = m_symmetryLineEdit->value();

	QString fshape = m_filterShapeBox->currentText();
	if (fshape == "gaussian") {
		genome_ptr->spatial_filter_select = flam3_gaussian_kernel;

	} else if (fshape == "hermite") {
		genome_ptr->spatial_filter_select = flam3_hermite_kernel;

	} else if (fshape == "box") {
		genome_ptr->spatial_filter_select = flam3_box_kernel;

	} else if (fshape == "triangle") {
		genome_ptr->spatial_filter_select = flam3_triangle_kernel;

	} else if (fshape == "bell") {
		genome_ptr->spatial_filter_select = flam3_bell_kernel;

	} else if (fshape == "bspline") {
		genome_ptr->spatial_filter_select = flam3_b_spline_kernel;

	} else if (fshape == "mitchell") {
		genome_ptr->spatial_filter_select = flam3_mitchell_kernel;

	} else if (fshape == "blackman") {
		genome_ptr->spatial_filter_select = flam3_blackman_kernel;

	} else if (fshape == "catrom") {
		genome_ptr->spatial_filter_select = flam3_catrom_kernel;

	} else if (fshape == "hanning") {
		genome_ptr->spatial_filter_select = flam3_hanning_kernel;

	} else if (fshape == "hamming") {
		genome_ptr->spatial_filter_select = flam3_hamming_kernel;

	} else if (fshape == "lanczos3") {
		genome_ptr->spatial_filter_select = flam3_lanczos3_kernel;

	} else if (fshape == "lanczos2") {
		genome_ptr->spatial_filter_select = flam3_lanczos2_kernel;

	} else if (fshape == "quadratic") {
		genome_ptr->spatial_filter_select = flam3_quadratic_kernel;

	} else {
		genome_ptr->spatial_filter_select = flam3_gaussian_kernel;

	}

	emit dataChanged();
}




