/***************************************************************************
 *   Copyright (C) 2007 by David Bitseff                                   *
 *   dbitsef@zipcon.net                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "mainwindowadapter.h"
#include "logger.h"

Lua::MainWindowAdapter::MainWindowAdapter(MainWindow* mw, LuaThread* t,
										  QObject* parent)
 : QObject(parent), m_win(mw), m_thread(t)
{
	logFine("Lua::MainWindowAdapter::MainWindowAdapter : const");
	moveToThread(t);
	connect(m_win, SIGNAL(mainWindowChanged()),
			this, SLOT(mainWindowChangedSlot()), Qt::QueuedConnection);
	connect(m_win->renderThread(), SIGNAL(flameRendered(RenderEvent*)),
			this, SLOT(flameRenderedSlot(RenderEvent*)), Qt::QueuedConnection);
	connect(this, SIGNAL(loadFileSignal(const QString&)),
			m_win, SLOT(loadFile(const QString&)), Qt::QueuedConnection);
	connect(m_thread, SIGNAL(scriptStopped()),
			this, SLOT(mainWindowChangedSlot()), Qt::QueuedConnection);
}


Lua::MainWindowAdapter::~MainWindowAdapter()
{
	logFine("Lua::MainWindowAdapter::MainWindowAdapter : dest");
	disconnect(this, SIGNAL(loadFileSignal(const QString&)),
			m_win, SLOT(loadFile(const QString&)));
	disconnect(m_win->renderThread(), SIGNAL(flameRendered(RenderEvent*)),
			this, SLOT(flameRenderedSlot(RenderEvent*)));
	disconnect(m_win, SIGNAL(mainWindowChanged()),
			this, SLOT(mainWindowChangedSlot()));
	disconnect(m_thread, SIGNAL(scriptStopped()),
			this, SLOT(mainWindowChangedSlot()));
}


GenomeVector* Lua::MainWindowAdapter::genomeVector()
{
	return m_win->genomeVector();
}

BasisTriangle* Lua::MainWindowAdapter::basisTriangle()
{
	return m_win->xformEditor()->basis();
}

void Lua::MainWindowAdapter::renderPreview(int idx)
{
	logFine("Lua::MainWindowAdapter::renderPreview");
	m_win->renderPreview(idx);
	waitForEvent();
}

bool Lua::MainWindowAdapter::loadFile(const QString& name)
{
	logFine("Lua::MainWindowAdapter::loadFile");
	emit loadFileSignal(name);
	waitForEvent();
	return true;
}

bool Lua::MainWindowAdapter::saveFile(const QString& name)
{
	return m_win->saveFile(name);
}

bool Lua::MainWindowAdapter::saveImage(const QString& name, int idx)
{
	logFine("Lua::MainWindowAdapter::saveImage");
	bool n = m_win->saveImage(name, idx);
	waitForEvent();
	return n;
}

bool Lua::MainWindowAdapter::save()
{
	return m_win->save();
}

void Lua::MainWindowAdapter::flameRenderedSlot(RenderEvent* e)
{
	logFine("Lua::MainWindowAdapter::flameRenderedSlot : signaling wait event");
	if (m_mutex.tryLock())
		m_mutex.unlock();
	else
		m_mutex.unlock();
}

void Lua::MainWindowAdapter::mainWindowChangedSlot()
{
	logFine("Lua::MainWindowAdapter::mainWindowChangedSlot : signaling wait event");
	if (m_mutex.tryLock())
		m_mutex.unlock();
	else
		m_mutex.unlock();
}

void Lua::MainWindowAdapter::waitForEvent()
{
	m_mutex.lock();
	while(!m_mutex.tryLock())
	{
		logFinest("Lua::MainWindowAdapter::waitForEvent : waiting");
		m_thread->msleep(10);
		QCoreApplication::processEvents();
	}
	m_mutex.unlock();
}



Lua::LuaThread* Lua::MainWindowAdapter::thread() const
{
	return m_thread;
}


MainWindow* Lua::MainWindowAdapter::window() const
{
	return m_win;
}
