/***************************************************************************
 *   Copyright (C) 2007 by David Bitseff                                   *
 *   dbitsef@zipcon.net                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "trianglesubset.h"

TriangleSubset::TriangleSubset(FigureEditor* f, BasisTriangle* b)
	: QList<Triangle*>(), QGraphicsPolygonItem(QPolygonF(QRectF(0.,0.,1.,1.))),
		m_editor(f), m_basis(b)
{
}

TriangleSubset::~TriangleSubset()
{
}

int TriangleSubset::type() const
{
	return RTTI;
}

void TriangleSubset::moveBy( double dx, double dy )
{
	QList<Triangle*>::const_iterator i;
	for (i = constBegin(); i != constEnd(); ++i)
		(*i)->moveBy(dx,dy);
	QTransform trans = transform();
	translate(dx, dy);
	QPolygonF pa;
	foreach (QPointF p, polygon())
		pa << mapToScene(p);
	setTransform(trans);
	setPolygon(pa);
}


void TriangleSubset::rotate( double rad, QPointF cpos )
{
	QList<Triangle*>::const_iterator i;
	for (i = constBegin(); i != constEnd(); ++i)
		(*i)->rotate(rad, (*i)->mapFromItem(this, cpos));
	QTransform trans = transform();
	translate(cpos.x(), cpos.y());
	QGraphicsPolygonItem::rotate(-rad);
	translate(-cpos.x(), -cpos.y());
	QPolygonF pa;
	foreach (QPointF p, polygon())
		pa << mapToScene(p);
	setTransform(trans);
	setPolygon(pa);
}

void TriangleSubset::scale( double dx, double dy, QPointF cpos )
{
	QList<Triangle*>::const_iterator i;
	for (i = constBegin(); i != constEnd(); ++i)
		(*i)->scale(dx, dy, (*i)->mapFromItem(this, cpos));
	QTransform trans = transform();
	translate(cpos.x(), cpos.y());
	QGraphicsPolygonItem::scale(dx, dy);
	translate(-cpos.x(), -cpos.y());
	QPolygonF pa;
	foreach (QPointF p, polygon())
		pa << mapToScene(p);
	setTransform(trans);
	setPolygon(pa);
}

void TriangleSubset::flipHorizontally(QPointF cpos)
{
	QList<Triangle*>::const_iterator i;
	for (i = constBegin(); i != constEnd(); ++i)
		(*i)->flipHorizontally((*i)->mapFromItem(this, cpos));

	double tx = cpos.x();
	QPolygonF pa;
	foreach (QPointF p, polygon())
	{
		p.rx()  =  p.x() - 2.*(p.x() - tx) ;
		pa << p;
	}
	setPolygon(pa);
}

void TriangleSubset::flipVertically(QPointF cpos)
{
	QList<Triangle*>::const_iterator i;
	for (i = constBegin(); i != constEnd(); ++i)
		(*i)->flipVertically((*i)->mapFromItem(this, cpos));

	double ty = cpos.y();
	QPolygonF pa;
	foreach (QPointF p, polygon())
	{
		p.ry()  =  p.y() - 2.*(p.y() - ty) ;
		pa << p;
	}
	setPolygon(pa);
}

