/***************************************************************************
 *   Copyright (C) 2007 by David Bitseff                                   *
 *   dbitsef@zipcon.net                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "undoring.h"
#include "logger.h"

UndoRing::UndoRing()
{
	current = head = -1;
	memset(ring, 0, sizeof(flam3_genome)*rsize);
}


UndoRing::~UndoRing()
{
	clear();
}

flam3_genome* UndoRing::next()
{
	if (current < head)
		current++;
	logFine(QString("UndoRing::next : current %1 / [%2,%3]")
			.arg(current)
			.arg(head - rsize)
			.arg(head));
	return ring + (current % rsize);
}

flam3_genome* UndoRing::prev()
{
	if (current > 0)
	{
		if (current > (head - rsize + 1))
			current--;
		logFine(QString("UndoRing::prev : current %1 / [%2,%3]")
			.arg(current)
			.arg(head - rsize)
			.arg(head));
		return ring + (current % rsize);
	}
	return ring;
}

void UndoRing::append(flam3_genome* t)
{
	current++;
	logFine(QString("UndoRing::append : current %1").arg(current % rsize));
	flam3_genome* g = ring + (current % rsize);
	flam3_copy(g, t);
	head = current;
}


void UndoRing::clear()
{
	logFine(QString("UndoRing::clear :"));
	for (int n = 0 ; n < rsize ; n++)
	{
		flam3_genome* g = ring + n;
		if (g && g->xform && g->num_xforms > 0)
			delete g->xform;
	}
	memset(ring, 0, rsize * sizeof(flam3_genome));
	current = head = -1;
}

bool UndoRing::atHead()
{
	return current == head;
}

bool UndoRing::atTail()
{
	return current == (head - rsize + 1) || current <= 0;
}


