/***************************************************************************
 *   Copyright (C) 2007 by David Bitseff                                   *
 *   dbitsef@zipcon.net                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef QOSMIC_XFEDIT_H
#define QOSMIC_XFEDIT_H

#include <QGraphicsScene>
#include <QGraphicsSceneMouseEvent>
#include <QGraphicsEllipseItem>
#include <QPainter>
#include <QMenu>
#include <QAction>
#include <QKeyEvent>


class BasisTriangle;
class Triangle;
class NodeItem;
class CoordinateMark;
class TriangleSubset;
class PostTriangle;
class GenomeVector;

typedef QList<Triangle*> TriangleList;

/**
 * The QGraphicsScene that manages the Triangles.
 */
class FigureEditor : public QGraphicsScene
{
	Q_OBJECT

	private:
		FigureEditor(QObject* parent=0);

	public:
		void clear();
		QString& getStatus();
		FigureEditor(GenomeVector*, QObject* parent=0);
		~FigureEditor();
		BasisTriangle* basis() const;
		const QMatrix& transform() const;
		CoordinateMark* mark() const;
		void setMarkVisible(bool);
		void setCoordTransform(QMatrix);
		void selectTriangle(Triangle*);
		void selectTriangle(int);
		void triangleModifiedAction(Triangle*);
		void setSceneRect(const QRectF&);
		int selectedTriangleIndex();
		Triangle* getSelectedTriangle();
		int getNumberOfTriangles();
		void setGridVisible(bool);
		bool gridVisible() const;
		QColor gridColor() const;
		void setGridColor(const QColor&);
		QColor bgColor() const;
		void setbgColor(QColor);
		void scaleBasis(double dx, double dy);
		bool hasSelection() const;
		TriangleSubset* selection() const;
		PostTriangle* post() const;
		void moveTriangleBy(double dx, double dy, Triangle* t);
		void moveSelectionBy(double dx, double dy);
		bool postEnabled();
		const TriangleList& triangles() const;
		QString getInfoLabel(Triangle*);
		void reset();

	signals:
		void coordinateChangeSignal(double, double);
		void triangleSelectedSignal(Triangle*);
		void triangleModifiedSignal(Triangle*);

	public slots:
		void addTriangleAction();
		void removeTriangleAction();
		void duplicateTriangleAction();
		void resetTriangleCoordsAction();
		void colorChangedAction(double);
		void selectNextTriangle();
		void selectPreviousTriangle();
		void autoScale();

		// graphicsitem horiz and vert flipping
		void flipTriangleHAction();
		void flipTriangleVAction();
		void flipTriangleHAction(QPointF);
		void flipTriangleVAction(QPointF);
		void flipSelectionHAction(QPointF);
		void flipSelectionVAction(QPointF);


		// graphicsitem rotate and scale methods
		void rotateTriangle(double, QPointF, Triangle*);
		void rotateSelection(double, QPointF);
		void scaleTriangle(double, double, QPointF, Triangle*);
		void scaleSelection(double, double, QPointF);

		// display and enable editing of the selected triangle's post coords
		void editPostTriangle(bool);

	protected:
		void mousePressEvent(QGraphicsSceneMouseEvent*);
		void mouseReleaseEvent(QGraphicsSceneMouseEvent*);
		void mouseMoveEvent(QGraphicsSceneMouseEvent*);
		void wheelEvent(QGraphicsSceneWheelEvent* wheelEvent);
		void drawBackground(QPainter*, const QRectF&);
		void keyPressEvent(QKeyEvent*);
		Triangle* getCurrentOrSelected();

	private slots:
		void triangleMenuAction(QAction*);

	private:
		QAbstractGraphicsShapeItem* moving;
		QPointF moving_start;
		QGraphicsSimpleTextItem* infoItem;
		TriangleList trianglesList;
		GenomeVector* genome;
		BasisTriangle* basisTriangle;
		QGraphicsSimpleTextItem* bLabelA;
		QGraphicsSimpleTextItem* bLabelB;
		QGraphicsSimpleTextItem* bLabelC;
		Triangle* selectedTriangle;
		CoordinateMark* coordinateMark;
		QMenu* popupMenu;
		QMenu* triangleMenu;
		QAction* addAction;
		QAction* dupAction;
		QAction* delAction;
		QAction* resAction;
		QAction* flipHAction;
		QAction* flipVAction;
		QAction* rescaleAction;
		bool grid_visible;
		QColor grid_color;
		QColor bg_color;
		TriangleSubset* selectionItem;
		PostTriangle* postTriangle;
		bool has_selection;
		bool is_selecting;
		bool editing_post;

};

#include "genomevector.h"
#include "basistriangle.h"
#include "triangle.h"
#include "nodeitem.h"
#include "coordinatemark.h"
#include "trianglesubset.h"
#include "posttriangle.h"

#endif

